@@ -1,34 +1,93 @@
-"""A full cache system written on top of Django's rudimentary one."""
+"""A full cache system written on top of Django's rudimentary one.
+
+Frequently used functions are:
+    cache_set(*keys, **kwargs)
+    cache_get(*keys, **kwargs)
+    cache_delete(*keys, **kwargs)
+keys.. parameters of general type which are convertable to string or hashable unambiguously.
+The keys can be of any general type which is convertable to string unambiguously or hashable.
+Every unknown kwarg is interpreted like two aditional keys: (key, val).
+Example:
+    cache_set('product', 123, value=product)
+    # is the same as
+    cache_set('product::123', value=product)
+
+More info below about parameters.
+"""
+# For keyedcache developers:
+# No additional keyword parameters should be added to the definition of
+# cache_set, cache_get, cache_delete or cache_key in the future.
+# Otherwise you must know what are you doing! Any appplication that would use
+# a new parameter will must check on startup that keyedcache is not older than
+# a required version !!(Otherwise kwargs unknown by an old version keyedcache
+# will be used as keys and cache_set/cache_get will use different keys that
+# would cause serious problems.)
 
 from django.conf import settings
-from django.core.cache import cache
+from django.core.cache import get_cache, InvalidCacheBackendError, DEFAULT_CACHE_ALIAS
+from django.core.exceptions import ImproperlyConfigured
 from django.utils.encoding import smart_str
-from django.utils.hashcompat import md5_constructor
+from django.utils.log import NullHandler
+from hashlib import md5
 from keyedcache.utils import is_string_like, is_list_or_tuple
+from warnings import warn
 import cPickle as pickle
 import logging
 import types
 
-log = logging.getLogger('keyedcache')
+log = logging.getLogger(__name__)
+log.setLevel(logging.INFO)
+log.addHandler(NullHandler())
 
+# The debugging variable CACHED_KEYS is exact only with the the Django
+# debugging server (or any single worker process server) and without restarting
+# the server between restarts of the main cache (memcached).
+# Keys in CACHED_KEYS variable never expire and can eat much memory on long
+# running servers. Currently it is not confirmed in Satchmo.
+# If more worker processes are used, the reported values of the following three
+# variables can skip randomly upwards downwards.
 CACHED_KEYS = {}
 CACHE_CALLS = 0
 CACHE_HITS = 0
+
 KEY_DELIM = "::"
 REQUEST_CACHE = {'enabled' : False}
-try:
-    CACHE_PREFIX = settings.CACHE_PREFIX
-except AttributeError:
-    CACHE_PREFIX = str(settings.SITE_ID)
-    log.warn("No CACHE_PREFIX found in settings, using SITE_ID.  Please update your settings to add a CACHE_PREFIX")
 
-try:
-    CACHE_TIMEOUT = settings.CACHE_TIMEOUT
-except AttributeError:
-    CACHE_TIMEOUT = 0
-    log.warn("No CACHE_TIMEOUT found in settings, so we used 0, disabling the cache system.  Please update your settings to add a CACHE_TIMEOUT and avoid this warning.")
+cache, cache_alias, CACHE_TIMEOUT, _CACHE_ENABLED = 4 * (None,)
+
+def keyedcache_configure():
+    "Initial configuration (or reconfiguration during tests)."
+    global cache, cache_alias, CACHE_TIMEOUT, _CACHE_ENABLED
+    cache_alias = getattr(settings, 'KEYEDCACHE_ALIAS', DEFAULT_CACHE_ALIAS)
+    try:
+        cache = get_cache(cache_alias)
+    except InvalidCacheBackendError:
+        log.warn("Warning: Could not find backend '%s': uses %s" % (cache_alias, DEFAULT_CACHE_ALIAS))
+        cache_alias = DEFAULT_CACHE_ALIAS  # it is 'default'
+        from django.core.cache import cache
+
+
+    CACHE_TIMEOUT = cache.default_timeout
+    if CACHE_TIMEOUT == 0:
+        log.warn("disabling the cache system because TIMEOUT=0")
+        
+    _CACHE_ENABLED = CACHE_TIMEOUT > 0 and not cache.__module__.endswith('dummy')
+
+    if not cache.key_prefix and (hasattr(settings, 'CACHE_PREFIX') or settings.SITE_ID != 1):
+        if hasattr(settings, 'CACHE_PREFIX'):
+            warn("The setting `CACHE_PREFIX` is obsoleted and is ignored by keyedcache.\n"
+                 """Use "CACHES = {'default': {... 'KEY_PREFIX': '...'}}" instead.""")
+        if settings.SITE_ID != 1:
+            hint = ("Use \"CACHES = {'default': {... 'KEY_PREFIX': '...'}}\" in order to\n"
+                    "differentiate caches or to explicitely confirm they should be shared.\n"
+                    " An easy solution is \"'CACHE_PREFIX': str(settings.SITE_ID)\".")
+            warn("An explicit KEY_PREFIX should be defined if you use multiple sites.\n%s" % hint)
+        if not cache.__module__.split('.')[-1] in ('locmem', 'dummy'):
+            raise ImproperlyConfigured(
+                    "Setting KEY_PREFIX is obligatory for production caches. See the previous warning.")
+
+keyedcache_configure()
 
-_CACHE_ENABLED = CACHE_TIMEOUT > 0
 
 class CacheWrapper(object):
     def __init__(self, val, inprocess=False):
@@ -62,11 +121,28 @@ class CacheNotRespondingError(Exception):
     pass
 
 def cache_delete(*keys, **kwargs):
+    """
+    Deletes the object identified by all ``keys`` from the cache.
+
+    keys:
+        Parameters of general type which are convertable to string or hashable
+        unambiguously.
+    kwargs:
+        children:
+            If it is True more objects starting with these keys are deleted.
+        other kwargs:
+            Unknown key=val is interpreted like two aditional keys: (key, val)
+
+    If no keys are present, all cached objects are to be deleted.
+    Deleting multiple multiple or all objects is usually not complete if the
+    project is running with multiple worker processes.
+    (It is reliable e.g. with a development server.)
+    """
     removed = []
     if cache_enabled():
         global CACHED_KEYS
         log.debug('cache_delete')
-        children = kwargs.pop('children',False)
+        children = kwargs.pop('children', False)
 
         if (keys or kwargs):
             key = cache_key(*keys, **kwargs)
@@ -167,6 +243,17 @@ def cache_function(length=CACHE_TIMEOUT):
 
 
 def cache_get(*keys, **kwargs):
+    """
+    Gets the object identified by all ``keys`` from the cache.
+
+    kwargs:
+        default:
+            Default value used if the object is not in the cache. If the object
+            is not found and ``default`` is not set or is None, the exception
+            ``NotCachedError`` is raised with the attribute ``.key = keys``.
+        other kwargs:
+            Unknown key=val is interpreted like two aditional keys: (key, val)
+    """
     if kwargs.has_key('default'):
         default_value = kwargs.pop('default')
         use_default = True
@@ -220,7 +307,18 @@ def cache_get(*keys, **kwargs):
 
 
 def cache_set(*keys, **kwargs):
-    """Set an object into the cache."""
+    """Set the object identified by all ``keys`` into the cache.
+
+    kwargs:
+        value:
+            The object to be cached.
+        length:
+            Timeout for the object. Default is CACHE_TIMEOUT.
+        skiplog:
+            If it is True the call is never logged. Default is False.
+        other kwargs:
+            Unknown key=val is interpreted like two aditional keys: (key, val)
+    """
     if cache_enabled():
         global CACHED_KEYS, REQUEST_CACHE
         obj = kwargs.pop('value')
@@ -246,40 +344,24 @@ def _hash_or_string(key):
         except AttributeError:
             return md5_hash(key)
 
-def cache_contains(*keys, **kwargs):
-    key = cache_key(keys, **kwargs)
-    return CACHED_KEYS.has_key(key)
-
 def cache_key(*keys, **pairs):
     """Smart key maker, returns the object itself if a key, else a list
     delimited by ':', automatically hashing any non-scalar objects."""
 
-    if is_string_like(keys):
-        keys = [keys]
-
-    if is_list_or_tuple(keys):
-        if len(keys) == 1 and is_list_or_tuple(keys[0]):
-            keys = keys[0]
-    else:
-        keys = [md5_hash(keys)]
+    if len(keys) == 1 and is_list_or_tuple(keys[0]):
+        keys = keys[0]
 
     if pairs:
         keys = list(keys)
-        klist = pairs.keys()
-        klist.sort()
-        for k in klist:
-            keys.append(k)
-            keys.append(pairs[k])
+        for k in sorted(pairs.keys()):
+            keys.extend((k, pairs[k]))
 
     key = KEY_DELIM.join([_hash_or_string(x) for x in keys])
-    prefix = CACHE_PREFIX + KEY_DELIM
-    if not key.startswith(prefix):
-        key = prefix+key
     return key.replace(" ", ".")
 
 def md5_hash(obj):
     pickled = pickle.dumps(obj, protocol=pickle.HIGHEST_PROTOCOL)
-    return md5_constructor(pickled).hexdigest()
+    return md5(pickled).hexdigest()
 
 
 def is_memcached_backend():
